<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 *
 *
 * @version    $Id: Articles.php 1629 2011-08-17 16:11:48Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 *
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for routing table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 *
 */
class Giscuit_Db_Table_LayerRouting extends Giscuit_Db_Table_Abstract
{
    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';
    
    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'layer_routing';
    
    /**
     * Algorithm names
     *  
     * @var array
     */
    public static $algorithmNames = array(
        1 => 'shortestPathDijkstra',
        2 => 'shortestPathAStar',
        3 => 'shortestPathShootingStar',
        
        4 => 'travelingSalesmanDijkstra' ,
        5 => 'travelingSalesmanAStar' ,
    );
   
    /**
     * Fetch the nearest edge
     * 
     * @param Zend_Db_Table $table
     * @param float $x North coordinate
     * @param float $y East coordinate
     * @return array
     */
    public static function nearestEdge ($table, $x, $y)
    {
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');
        
        if ($config->map->units == 'dd') {
            $offset = 0.01;
            $snapSize = 0.00001;
        } else {
            $offset = 1110;
            $snapSize = 1.11;
        }
        
        $tableSettings = $table->getSettings();
        $theGeomField = $table->getAdapter()->quoteIdentifier($tableSettings['table'] . '.the_geom');
        $distance = "ST_Distance({$theGeomField}, ST_GeometryFromText('POINT({$x} {$y})', {$config->map->srid}))";
        $where = "{$theGeomField} && ST_SetSRID('BOX(
            " . ($x - $offset) . " 
            " . ($y - $offset) . ", 
            " . ($x + $offset) . " 
            " . ($y + $offset) . ")'::box2d, {$config->map->srid}) ";
        
        $select = $table->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'gid',
            'source',
            'target',
            'distance' => new Zend_Db_Expr($distance),
            'the_geom' => new Zend_Db_Expr("ST_AsText(ST_SnapToGrid({$theGeomField}, $snapSize))")
        );
        
        $select->from($config->database->schema . '_routing.' . $tableSettings['table'], $fields);
        $select->where($where);
        $select->order('distance');
        $select->limit(1);
        
        $result = $table->fetchRow($select)->toArray();
        
        $result['x'] = $x;
        $result['y'] = $y;
        
        return $result;
    }
    
    /**
     * Get routing path
     * 
     * @param Zend_Db_Table $table
     * @param array $startEdge Starting edge
     * @param array $endEdge Ending edge
     * @param integer $algorithm Routing aglorithm:
     * <ul><li>1 - shortest_path</li>
     * <li>2 - shortest_path_astar</li>
     * <li>3 - shortest_path_shooting_star</li>
     * <li>3 - shortest_path_shooting_star</li></ul>
     * @param bool $dir Is the grapth directed
     * @param bool $rc Is the reserve cost calculated
     * @param string $costCoef Cost coefficient column name
     * @param string $reverseCostCoef Reverse cost coefficient column name
     * 
     * @return array
     */
    public static function getPath($table, $startEdge, $endEdge, $algorithm, 
        $dir = false, 
        $rc = false,
        $costCoef = '',
        $reverseCostCoef = ''
    )        
    {
        $dirQue = $dir == true ? 'true' : 'false';
        $rcQue = $rc == true ? 'true' : 'false';
        $maxCost = 100000000;
        
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
                
        $tableName = $table->getAdapter()->quoteIdentifier($table->info('name')); 
        $tableSchema = $table->getAdapter()->quoteIdentifier($config->database->schema . '_routing');
        
        if ($config->map->units == 'dd') {
            $offset = 0.1;
            $snapSize = 0.00001;
        } else {
            $offset = 11100;
            $snapSize = 1.11;
        }
        
        $start = $startEdge['source'];
        $end = $endEdge['target'];
        //FIXME:
        if($start == $end) {
            $start = $startEdge['target'];
            $end = $endEdge['source'];
        }
        
        //Bbox
        $bboxSelect = "SELECT ST_Expand(ST_Extent(the_geom)::box2d, {$offset})::box2d AS bbox " .
            "FROM {$tableSchema}.{$tableName} WHERE gid = {$startEdge['gid']} OR gid =  {$endEdge['gid']}";
        $bbox = $db->fetchRow($bboxSelect);
        if (is_array($bbox)) {
            $bbox = $bbox['bbox'];
        } else {
            $bbox = $bbox->bbox;
        }

        //
        $select = $db->select();
        $select->from($config->database->schema . '_routing.' . $table->info('name'), array(
                'gid',
                'the_geom' => new Zend_Db_Expr("ST_AsText(ST_SnapToGrid(the_geom, $snapSize))")
            )
        );
        
        //Route select
        $routeSelect = $db->select();
        
        //Route fields
        $routeFields = array(
            'id' => 'gid' ,
            'source' ,
            'target' ,
            'cost' => new Zend_Db_Expr("{$tableName}.cost")
        );
        
        //Add reverse cost field
        if($rc) {
            $routeFields['reverse_cost'] = new Zend_Db_Expr("{$tableName}.reverse_cost");
        }
        
        //Route algorithm
        if ($algorithm == 1) {
            $algorithmQue = 'shortest_path';
        } else if ($algorithm == 2) {
            $algorithmQue = 'shortest_path_astar';
            $routeFields = array_merge($routeFields, array(
                'x1' ,
                'y1' ,
                'x2' ,
                'y2'
            ));
        } else if ($algorithm == 3) {
            $algorithmQue = 'shortest_path_shooting_star';
            $routeFields = array_merge($routeFields, array(
                'x1' ,
                'y1' ,
                'x2' ,
                'y2' ,
                'rule' ,
                'to_cost'
            ));
            $start = $startEdge['gid'];
            $end = $endEdge['gid'];
        }
        
        //Add route cost coefficients
        if($costCoef || $reverseCostCoef) {
            if($costCoef) {
                $routeFields['cost'] = new Zend_Db_Expr("{$tableName}.cost * cost_table.\"{$costCoef}\"");
            }
            
            if($rc && $reverseCostCoef) {
                $routeFields['reverse_cost'] = new Zend_Db_Expr("{$tableName}.reverse_cost * cost_table.\"{$reverseCostCoef}\"");
            }
            
            $routeSelect->joinLeft(
                array(
                    'cost_table' => $table->info('schema') . '.' . $table->info('name')
                ),
                "cost_table.gid = {$tableName}.parent_gid",
                array()
            );
        }
        
        //If route cost is negative set maximum cost
        $routeFields['cost'] = new Zend_Db_Expr("CASE WHEN {$routeFields['cost']} < 0 THEN {$maxCost} ELSE {$routeFields['cost']} END");
        if($rc) {
            $routeFields['reverse_cost'] = new Zend_Db_Expr("CASE WHEN {$routeFields['reverse_cost']} < 0 THEN {$maxCost} ELSE {$routeFields['reverse_cost']} END");
        }    
                
        //Route from
        $routeSelect->from(
            $config->database->schema . '_routing.' . $table->info('name'), 
            $routeFields
        );
        
        //Route bbox
        $routeSelect->where("{$tableName}.the_geom && ST_SetSRID(\'{$bbox}\'::box2d, {$config->map->srid})");
        
        //
        $select->join(
            array(
                'route_table' => 
                    new Zend_Db_Expr("{$algorithmQue}('{$routeSelect}', {$start}, {$end}, {$dirQue}, {$rcQue})"),
            ),
            "route_table.edge_id = {$tableName}.gid",
            array()
        );

        //
        try {
            $currFetchMode = $db->getFetchMode();
            $db->setFetchMode(Zend_Db::FETCH_ASSOC);
            $result = $db->fetchAll($select);
            $db->setFetchMode($currFetchMode);
        } catch (Exception $e) {
            $result = array();
        }

        //
        if (count($result)) {
            // 
            $startEdgeSimplified = array(
                'gid' => $startEdge['gid'],
                'the_geom' => $startEdge['the_geom']
            );
            $endEdgeSimplified = array(
                'gid' => $endEdge['gid'],
                'the_geom' => $endEdge['the_geom']
            );
            array_unshift($result, $startEdgeSimplified);
            array_push($result, $endEdgeSimplified);
                                    
            //Delete duplicates
            $gidList = array();
            foreach ($result as $key => $value) {
                if(in_array($value['gid'], $gidList)) {
                    unset($result[$key]);    
                } else {
                    array_push($gidList, $value['gid']);
                }
            }
            $result = array_values($result);
            
            //WKT to Array
            $geometries = array();
            for ($i=0; $i < count($result); $i++) {
                $geometry = substr($result[$i]['the_geom'], 17);
                $geometry = substr($geometry, 0, -2);
                $geometry = explode(',', $geometry);
                array_push($geometries, $geometry);
            }
            
            //Align
            if($geometries[0][0] == $geometries[1][0] || 
                $geometries[0][0] == $geometries[1][count($geometries[1]) - 1]) {
                $geometries[0] = array_reverse($geometries[0]);
            }
            
            //Delete last point from first line
            $linePoints = $geometries[0];
            unset($linePoints[count($linePoints) - 1]);
            
            for ($i=1; $i < count($geometries); $i++) {
                //Check if last point of previous line matches last point of
                //current line and if yes reverse the order
                   $geomA = explode(' ', $geometries[$i - 1][count($geometries[$i - 1]) - 1]);
                $geomB = explode(' ', $geometries[$i][count($geometries[$i]) - 1]);
                if(strlen($geomA[0]) > strlen($geomB[0])) {
                    $geomA[0] = substr($geomA[0], 0, strlen($geomB[0]));
                } else if(strlen($geomB[0]) > strlen($geomA[0])) {
                    $geomB[0] = substr($geomB[0], 0, strlen($geomA[0]));
                }
                if(strlen($geomA[1]) > strlen($geomB[1])) {
                    $geomA[1] = substr($geomA[1], 0, strlen($geomB[1]));
                } else if(strlen($geomB[1]) > strlen($geomA[1])) {
                    $geomB[1] = substr($geomB[1], 0, strlen($geomA[1]));
                }
                if($geomA[0] == $geomB[0] && $geomA[1] == $geomB[1]) {
                    $geometries[$i] = array_reverse($geometries[$i]);
                }
                
                //Delete last point from current line if it's not the last line
                if(count($geometries) - 1 == $i ) {
                    $jOffset = 0;
                } else {
                    $jOffset = 1;
                }
                for ($j=0; $j < count($geometries[$i]) - $jOffset; $j++) {
                    $linePoints[] = $geometries[$i][$j];
                }
            }
            
            //
            for ($i=0; $i < count($result); $i++) {
                $result[$i]['the_geom'] = 'LINESTRING(' . implode(',', $geometries[$i]) . ')';
            }
                        
            //Merge
            $linePointsStm =  "LINESTRING(" . implode(',', $linePoints) . ")";
            $p1Stm = "ST_Line_Locate_Point(the_geom, ST_MakePoint({$startEdge['x']}, {$startEdge['y']}))";
            $p2Stm = "ST_Line_Locate_Point(the_geom, ST_MakePoint({$endEdge['x']}, {$endEdge['y']}))";
            $pathStm = "SELECT ST_AsText(the_geom) as the_geom, {$p1Stm} AS p1, {$p2Stm} AS p2 FROM (";
            $pathStm .= "SELECT ST_GeometryFromText(:the_geom) as the_geom";
            $pathStm .= ') as stm1';
            $pathStmObj = $table->getAdapter()->prepare($pathStm);
            $pathStmObj->bindParam('the_geom', $linePointsStm, PDO::PARAM_STR);
            $pathStmObj->execute();
            $pathResult = $pathStmObj->fetch(PDO::FETCH_OBJ);
                        
            //Trim
            if($pathResult->p1 > $pathResult->p2) {
                $tmpPoint = $pathResult->p1;
                $pathResult->p1 = $pathResult->p2;
                $pathResult->p2 = $tmpPoint;
            }
            $trimedPathStm = "SELECT ST_AsText(ST_Line_Substring(ST_GeometryFromText(:the_geom), {$pathResult->p1}, {$pathResult->p2})) AS the_geom";
            $trimedPathStmObj = $table->getAdapter()->prepare($trimedPathStm);
            $trimedPathStmObj->bindParam('the_geom', $pathResult->the_geom, PDO::PARAM_STR);
            $trimedPathStmObj->execute();
            $trimedPathResult = $trimedPathStmObj->fetch(PDO::FETCH_OBJ)->the_geom;

            //Result
            return array(
                'path' => $trimedPathResult
            );
        } else {
            return array();
        }
    }
    
    /**
     * Order edges using traveling salesman problem algorithm
     * 
     * @param Zend_Db_Table $table
     * @param array $edges Edges
     * @return array
     */
    public static function orderEdgesByTsp($table, $edges) 
    {
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
                
        $tableName = $table->getAdapter()->quoteIdentifier($table->info('name')); 
        $tableSchema = $table->getAdapter()->quoteIdentifier($config->database->schema . '_routing');
        
        //Vertices
        $vertices = array();
        foreach ($edges as $key => $value) {
            $vertices[] = $value['source'];
        }
        $vertices = implode(',', array_unique($vertices));
        
        //Select
        $select = $db->select();
        
        //Route select
        $routeSelect = $db->select();
        
        //FIXME: With distinct we get errors on routes with 3 vertices
        //$routeSelect->distinct();
        
        //Route fields
        $routeFields = array(
            'source_id' => 'source',
            'x' => 'x1' ,
            'y' => 'y1',
        );
        
        //Route from
        $routeSelect->from(
            $config->database->schema . '_routing.' . $table->info('name'), 
            $routeFields
        );
        
        //Route bbox
        $routeSelect->where("{$tableName}.source IN ({$vertices})");
        
        //From
        $select->from(new Zend_Db_Expr("tsp('{$routeSelect}', '{$vertices}', {$edges[0]['source']})"), 
            array(
                'vertex_id',
            )
        );
        
        //Result
        $result = $db->fetchCol($select);
        
        //Order
        $order = array();
        for($i=0; $i < count($result); $i++) {
            foreach ($edges as $key => $value) {
                if($value['source'] ==  $result[$i]) {
                    $edges[$key]['order'] = $i;
                }
            }
        }
        foreach ($edges as $key => $value) {
            $order[$key] = $value['order'];
        }
        array_multisort($order, SORT_ASC, SORT_NUMERIC, $edges);
        
        //
        return $edges;
    }
    
}