<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Vector.php 3198 2012-12-06 12:33:28Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Map_Layer_Abstract
 */
require_once 'Giscuit/Map/Layer/Abstract.php';

/**
 * Class for abstract mapserver vector layer object
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Map_Layer_Vector extends Giscuit_Map_Layer_Abstract
{

    /**
     * Database table
     *
     * @var Giscuit_Db_Table_Abstract
     */
    protected $_table;
    
    /**
     * MS_CJC
     *
     * @var array
     */
    public static $msCjc = array(
        1 => 'bevel',
        2 => 'butt',
        3 => 'mitter',
        4 => 'round',
        5 => 'square',
        6 => 'triangle',
    );
    
    /**
     * Class constructor
     *
     * @param mixed $map Map object
     * @param array $settings Settings
     * @return void
     */
    public function __construct ($map, $settings)
    {
        parent::__construct($map, $settings);
        
        if(! empty($this->_settings['table'])) {
            //Postgis
            $this->setPostgisOptions();
        
            //Table
            $this->_createTable();
        }
        
        //Set classes and styles
        $this->setClassesAndStyles();
        
        //Processing options
        $this->setProcessing('LABEL_NO_CLIP=True');
        
        //History plugin
        $front = Zend_Controller_Front::getInstance();
        $editingPlugin = $front->getPlugin('Giscuit_Controller_Plugin_Editing');
        if($editingPlugin != false) {
            $editingPlugin->vectorLayerConstruct($this);
        }
        
    }

    /**
     * Set database table object based on layer class name
     * 
     * @return void
     */
    protected function _createTable ()
    {
        $className = 'Giscuit_Db_Table_Layer_' . $this->_settings['class'];
        
        //Table options
        $tableOptions = array(
            'name' => $this->_settings['table']
        );
        if(! empty(Zend_Registry::get('settings')->historyDateEnd)) {
            $tableOptions['withHistory'] = true;
        }
        
        //Select options
        $selectOptions = array(); 
        if(! empty($this->_settings['whereHistoryId'])) {
            $selectOptions['whereHistoryId'] = $this->_settings['whereHistoryId']; 
            $tableOptions['withHistory'] = true;      
        }
        
        $this->_table = new $className($tableOptions);
        $this->_table->layer = $this;
        $this->_table->setSettings($this->_settings);
                
        if($this->parent->dumpAllAttributes == false) {
            $select = $this->_table->getSelect('gid_geometry_label', $selectOptions);    
        } else {
            $select = $this->_table->getSelect('all', $selectOptions);
        }

        if (empty($select)) {
            $this->set('status', MS_OFF);
        } else {
            if(isset($this->_table->historyEnabled) && $this->_table->historyEnabled == true) {
                $this->setData($select, 'hgid');
            } else {
                $this->setData($select);    
            }
        }
    }

    /**
     * Get database table object
     */
    public function getTable ()
    {
        return $this->_table;
    }   

    /**
     * Set layer data property
     * 
     * @param string $select SQL query
     * @param string $unique Unique table column
     * @return void 
     */
    public function setData ($select, $unique = 'gid')
    {
        $config = Zend_Registry::get('config');
        
        $this->set('data', "the_geom from ($select) as tmp using SRID={$config->map->srid} using unique $unique");
    }

    /**
     * Sets the options for a postgis layer
     * 
     * @return void 
     */
    public function setPostgisOptions ()
    {
        $config = Zend_Registry::get('config');
        
        $connectionString = Giscuit_Controller_Plugin_Init::getDbConnectionString();
                
        $this->setConnectionType(MS_POSTGIS);    
        
        $this->setProcessing('CLOSE_CONNECTION=DEFER');
        $this->set('connection', $connectionString);
    }
    
    /**
     * Populate the label class with values
     * 
     * @param mixed $label
     * @param $options
     * @return void
     */
    public function setLabel($label, array $options = array())
    {
        //
        $msVersion = ms_GetVersionInt();
        
        //
        if($this->_settings['label'] == 'f' ||
            $this->parent->dumpAllAttributes == true ||
            empty($this->_settings['label_fields']) ||
            (
                isset($this->_settings['applyClassesStylesLabels']) && 
                $this->_settings['applyClassesStylesLabels'] == false
            )
            ) {
                
            return;
        }
        
        //Class label inactive
        if (! isset($this->_settings['classlabel_active']) || $this->_settings['classlabel_active'] == 'f') { 
            
            $options = $this->_settings;
            
        //Class label active
        } else {
            if(! isset($options['label_active']) || $options['label_active'] == 'f') {
                return;
            }
        }
                
        $color = self::hexToRgb($options['label_color']);
        
        $label->set('type', MS_TRUETYPE);
        $label->set('partials', MS_FALSE);
        
        //font
        $label->set('font', $options['label_font']);
        
        //size
        $label->set('size', $this->adjustSize($options['label_size']));
        
        //color
        $label->color->setrgb($color[0], $color[1], $color[2]);
        
        //position
        $label->set('position', $options['label_position']);
        
        //angle
        if($options['label_angle'] == 'f') {
            
        } elseif($options['label_angle'] == 'auto') {
            $this->setLabelAngleToAuto($label);
        } elseif($options['label_angle'] == 'follow') {
            $this->setLabelAngleToFollow($label);
        }
        
        //buffer
        if (! empty($options['label_buffer'])) {
            $label->set('buffer', $options['label_buffer']);
        }
        
        //backgroundcolor
        if(! empty($options['label_backgroundcolor'])) {
            $backgroundcolor = self::hexToRgb($options['label_backgroundcolor']);
            
            if(property_exists($label, 'backgroundcolor')) {
                $label->backgroundcolor->setrgb($backgroundcolor[0], $backgroundcolor[1], $backgroundcolor[2]);
            }
            
            //FIXME:
               if($msVersion >= 60000) {
                $label->updateFromString("LABEL STYLE GEOMTRANSFORM 'labelpoly' COLOR {$backgroundcolor[0]} {$backgroundcolor[1]} {$backgroundcolor[2]} OUTLINECOLOR 255 255 255 END END");
            }

        //outlinecolor
        } else {
            $label->outlinecolor->setrgb(255, 255, 255);    
        }
        
        //minfeaturesize
        if (! empty($options['label_minfeaturesize'])) {
            $label->set('minfeaturesize', $options['label_minfeaturesize']);
        } else {
            $label->set('minfeaturesize', MS_AUTO);
        }
        
        //mindistance
        if (! empty($options['label_mindistance'])) {
            $label->set('mindistance', $options['label_mindistance']);
        }
        
        //repeatdistance
        if (! empty($options['label_repeatdistance'])) {
            if(property_exists($label, 'repeatdistance')) {
                $label->set('repeatdistance', $options['label_repeatdistance']);
            }
        }
        
        //wrap
        if(! empty($options['label_wrap'])) {
            $label->set('wrap', ord($options['label_wrap']));
        }
        
        //align
        $label->set('align', $options['label_align']);
        
        //maxlength
        if(! empty($options['label_maxlength'])) {
            $label->set('maxlength', $options['label_maxlength']);
        }
        
        //priority
        if(! empty($options['label_priority'])) {
            $label->set('priority', $options['label_priority']);
        }
        
        //Layer
        $this->set('labelcache', MS_ON);
        $this->set('labelitem', '_label_');
        
        $this->_setLabelCallback($label);
    }

    /**
     * Is called after label propreties have been populated
     * @param mixed $class Class object
     * @return void
     */
    protected function _setLabelCallback ($label)
    {

    }

    /**
     * Converts a hex color definition into rgb
     * 
     * @param string $hex Color
     * @return string
     */
    public static function hexToRgb ($hex)
    {
        //Strip off any leading #
        if (0 === strpos($hex, '#')) {
            $hex = substr($hex, 1);
        } else {
            if (0 === strpos($hex, '&H')) {
                $hex = substr($hex, 2);
            }
        }
        
        //Break into hex 3-tuple
        $cutpoint = ceil(strlen($hex) / 2) - 1;
        $rgb = explode(':', wordwrap($hex, $cutpoint, ':', $cutpoint), 3);
        
        //Convert each tuple to decimal
        $rgb[0] = (isset($rgb[0]) ? hexdec($rgb[0]) : 0);
        $rgb[1] = (isset($rgb[1]) ? hexdec($rgb[1]) : 0);
        $rgb[2] = (isset($rgb[2]) ? hexdec($rgb[2]) : 0);
        
        return $rgb;
    }

    /**
     * Convert RGB to HEX
     * 
     * @param $r Red
     * @param $g Green
     * @param $b Blue
     * @return string
     */
    public static function rgb2Hex ($r, $g = -1, $b = -1)
    {
        if (is_array($r) && sizeof($r) == 3) {
            list ($r, $g, $b) = $r;
        }
        
        $r = intval($r);
        $g = intval($g);
        $b = intval($b);
        
        $r = dechex($r < 0 ? 0 : ($r > 255 ? 255 : $r));
        $g = dechex($g < 0 ? 0 : ($g > 255 ? 255 : $g));
        $b = dechex($b < 0 ? 0 : ($b > 255 ? 255 : $b));
        
        $color = (strlen($r) < 2 ? '0' : '') . $r;
        $color .= (strlen($g) < 2 ? '0' : '') . $g;
        $color .= (strlen($b) < 2 ? '0' : '') . $b;
        
        return '#' . $color;
    }

    /**
     * Correct the font size according to current dpi
     * 
     * @param float $fontSize Font size
     * @return void
     */
    public function adjustSize ($fontSize)
    { 
        return floatval($fontSize);
    }

    /**
     * Populate classes and styles objects from settings
     * 
     * @return void
     */
    public function setClassesAndStyles ()
    {
        //
        $msVersion = ms_GetVersionInt();
        
        //
        if(empty($this->_settings['classes']) 
            || (isset($this->_settings['applyClassesStylesLabels']) && $this->_settings['applyClassesStylesLabels'] == false)) {
                
            return;   
        }
        
        foreach ($this->_settings['classes'] as $class) {
            
            //Skip classes with empty name
            if(empty($class['name'])) {
                continue;
            }
            
            $classObj = new Giscuit_Map_Class($this, $class['name']);
            
            //Minscaledenom
            if (! empty($class['minscaledenom'])) {
                $classObj->set('minscaledenom', $class['minscaledenom']);
            }
            
            //Maxscaledenom
            if (! empty($class['maxscaledenom'])) {
                $classObj->set('maxscaledenom', $class['maxscaledenom']);
            }
            
            //Expression
            if (! empty($class['expression'])) {
                $classObj->setExpression($class['expression']);
            }
            
            //Expression
            if (! empty($class['legend'])) {
                $classObj->legend = $class['legend'] == 't' ? true : false;
            }
            
            foreach ($class['styles'] as $style) {
                $styleObj = new Giscuit_Map_Style($classObj);
                
                //Color
                if (! empty($style['color'])) {
                    $color = self::hexToRgb($style['color']);
                    $styleObj->color->setrgb($color[0], $color[1], $color[2]);
                }
                
                //Outlinecolor
                if (! empty($style['outlinecolor'])) {
                    $outlinecolor = self::hexToRgb($style['outlinecolor']);
                    $styleObj->outlinecolor->setrgb($outlinecolor[0], $outlinecolor[1], $outlinecolor[2]);
                }
                
                //Backgroundcolor
                if (! empty($style['backgroundcolor'])) {
                    $backgroundcolor = self::hexToRgb($style['backgroundcolor']);
                    $styleObj->backgroundcolor->setrgb($backgroundcolor[0], $backgroundcolor[1], $backgroundcolor[2]);
                }
                
                //Symbolname
                if (! empty($style['symbolname'])) {
                    $map = $this->parent;                    
                    $symbol = $map->getsymbolbyname($style['symbolname']);
                    
                    if($symbol != -1) {
                        $styleObj->set('symbolname', $style['symbolname']);
                    } else {
                        throw new Giscuit_Exception("Invalid symbol {$style['symbolname']} in class {$class['name']}");
                    }
                    
                    unset($symbol);
                }
                
                //Opacity
                if(property_exists($styleObj->getMsObj(), 'opacity')) {
                    $styleObj->set('opacity', intval($style['opacity']));
                }
                //FIXME:
                if($msVersion >= 60000) {
                    $styleObj->updateFromString("STYLE OPACITY {$style['opacity']} END");
                }

                //Width
                if (! empty($style['width'])) {
                    $styleObj->set('width', $this->adjustSize($style['width']));
                }
                
                //Size
                if (! empty($style['size'])) {
                    $styleObj->set('size', $this->adjustSize($style['size']));
                }
                
                //Angle
                if (! empty($style['angle'])) {
                    $styleObj->set('angle', $style['angle']);
                }
                
                //Pattern
                if (! empty($style['pattern'])) {
                    if(property_exists($styleObj->getMsObj(), 'pattern')) {
                        $pattern = explode(' ', $style['pattern']);
                        $styleObj->set('pattern', $pattern);
                    } else {
                        //FIXME:
                        if($msVersion >= 60000) {
                            $styleObj->updateFromString("STYLE PATTERN {$style['pattern']} END");
                        }
                    }
                }
                    
                //Position
                if (! empty($style['position'])) {
                    if(property_exists($styleObj->getMsObj(), 'position')) {
                        $styleObj->set('position', intval($style['position']));
                    } else {
                        //FIXME:
                        if($msVersion >= 60000) {
                            $styleObj->updateFromString("STYLE POSITION {$style['position']} END");
                        }
                    }
                }
                
                //Gap
                if (! empty($style['gap'])) {
                    if(property_exists($styleObj->getMsObj(), 'gap')) {
                        $styleObj->set('gap', floatval($style['gap']));
                    } else {
                        //FIXME:
                        if($msVersion >= 60000) {
                            $styleObj->updateFromString("STYLE GAP {$style['gap']} END");
                        }
                    }
                }
                
                //Linecap
                if (! empty($style['linecap'])) {
                    if(property_exists($styleObj->getMsObj(), 'linecap')) {
                        $styleObj->set('linecap', intval($style['linecap']));
                    } else {
                        //FIXME:
                        if($msVersion >= 60000) {
                            $styleObj->updateFromString('STYLE LINECAP ' . self::$msCjc[$style['linecap']] . ' END');
                        }
                    }
                }
                
                //Linejoin
                    if (! empty($style['linejoin'])) {
                    if(property_exists($styleObj->getMsObj(), 'linejoin')) {
                        $styleObj->set('linejoin', intval($style['linejoin']));
                    } else {
                        //FIXME:
                        if($msVersion >= 60000) {
                            $styleObj->updateFromString('STYLE LINEJOIN ' . self::$msCjc[$style['linejoin']] . ' END');
                        }
                    }
                }
                
                //Linejoinmaxsize 
                if (! empty($style['linejoinmaxsize'])) {
                    if(property_exists($styleObj->getMsObj(), 'linejoinmaxsize')) {
                        $styleObj->set('linejoinmaxsize', intval($style['linejoinmaxsize']));
                    } else {
                        //FIXME:
                        if($msVersion >= 60000) {
                            $styleObj->updateFromString("STYLE LINEJOINMAXSIZE {$style['linejoinmaxsize']} END");
                        }
                    }
                }
            }
            
            //Label
            if($msVersion >= 60200) {
                $label = new labelObj();
                
                if($this->_settings['classlabel_active'] == 't') {
                    if($class['label_active'] == 't') {
                        $classObj->addLabel($label);
                        $this->setLabel($label, $class);
                    }    
                } else {
                       $classObj->addLabel($label);
                       $this->setLabel($label, $class);
                }
                
            } else {
                $this->setLabel($classObj->label, $class);
            }
        }
    
    }
    
    /**
     * Set label angle to auto
     * 
     * @param mixed $label
     * @return void
     */
    public function setLabelAngleToAuto($label)
    {
        $msVersion = ms_GetVersionInt();
        
        if(property_exists($label, 'autoangle')) {
            $label->set('autoangle', MS_TRUE);
        }
        
        if($msVersion >= 60000) {
            $label->set('anglemode', MS_AUTO);
        }
    }
    
    /**
     * Set label angle to follow
     * 
     * @param mixed $label
     * @return void
     */
    public function setLabelAngleToFollow($label)
    {
        $msVersion = ms_GetVersionInt();
        
        if(property_exists($label, 'autoangle')) {
            $label->set('autoangle', MS_TRUE);
        }
        if(property_exists($label, 'autofollow')) {
            $label->set('autofollow', MS_TRUE);
        }
        
        if($msVersion >= 60000) {
            $label->set('anglemode', MS_FOLLOW);
        }
    }
    
    /**
     * Generate SLD
     * 
     * @param string $name Layer name
     * @return string
     */
    public function generateSLD ($name = '')
    {
        if ($name) {
            $lastName = $this->name;
            $this->getMsObj()->set('name', $name);
        }
        $this->set('status', MS_ON);
        
        $layerClassName = get_class($this);
        
        if (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Point')) {
            $layerType = 'Point';
        } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Line')) {
            $layerType = 'Line';
        } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Polygon')) {
            $layerType = 'Polygon';
        }
        
        //Well-known mark symbol
        if ($layerType == 'Point') {
            for ($a = 0; $a < $this->numclasses; $a ++) {
                $class = $this->getClass($a);
                for ($b = 0; $b < $class->numstyles; $b ++) {
                    $style = $class->getStyle($b);
                    
                    //Circle
                    if (stristr($style->symbolname, 'circle')) {
                        $style->set('symbolname', 'circle');
                    
                     //Square
                    } elseif (stristr($style->symbolname, 'square')) {
                        $style->set('symbolname', 'square');
                    
                    //Triangle
                    } elseif (stristr($style->symbolname, 'triangle')) {
                        $style->set('symbolname', 'triangle');
                    
                    //Star
                    } elseif (stristr($style->symbolname, 'star')) {
                        $style->set('symbolname', 'star');
                    
                    //Cross
                    } elseif (stristr($style->symbolname, 'cross')) {
                        $style->set('symbolname', 'cross');
                    
                    //Default
                    } else {
                        $style->set('symbolname', 'circle');
                    }
                
                }
            
            }
        }
        
        $sld = $this->getMsObj()->generateSLD();
        
        if ($name) {
            $this->getMsObj()->set('name', $lastName);
        }
        
        return $sld;
    }
    
    /**
     * Parse legend
     * 
     * @param array $data Data
     * @return array
     */
    public function parseLegend ($data)
    {
        return $data;
    }
}