/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Info.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */

//
Giscuit.Util.WebSql = {
    dbConn: null,
    name: 'gt',
    desc: 'Giscuit',
    size: 50 * 1024 * 1024,
    version: '1.0'    
};
if(typeof(window.openDatabase) === "function") {
    Giscuit.Util.Db = Giscuit.Util.WebSql;
}

/**
 * Class: OpenLayers.Protocol.SQL.WebSQL
 * 
 *  Inherits from:
 *   - <OpenLayers.Protocol.SQL>
 */
OpenLayers.Protocol.SQL.WebSQL = OpenLayers.Class(OpenLayers.Protocol.SQL, {
    /**
     * Property: maxId
     * {Integer} Used internally to keep track of max feature id.
     */
    maxId: 0,
    
    /**
     * Property: tableName
     * {String} Database name.  A table name is automatically generated if one
     *     is not provided in the protocol configuration.
     */
    tableName: null,
    
    /**
     * Property: db
     * {Database}
     */
    db: null,

    /**
     * Property: supported
     * {Boolean} The environment supports this protocol.
     */
    supported: (typeof window.openDatabase === "function"),

    /**
     * Constructor: OpenLayers.Protocol.WebSQL
     * Create a new protocol for reading and writing feature data using the
     *     Web SQL protocol (http://www.w3.org/TR/webdatabase/).
     *
     * Parameters:
     * options - {Object} Optional properties to be set on the protocol.
     */
    initialize: function(options) {
        if (this.supported) {
            OpenLayers.Protocol.prototype.initialize.apply(this, [options]);
            if (!this.format) {
                this.format = new OpenLayers.Format.GeoJSON();
            }
            if (!this.tableName) {
                this.tableName = OpenLayers.Util.createUniqueID("websql_");
            }
            this.initializeDatabase();
        }
    },
    
    /**
     * Method: destroy
     * Clean up.
     */
    destroy: function() {
        if (!this.options || !this.options.format) {
            this.format.destroy();
        }
        this.format = null;
        this.db = null;
        OpenLayers.Protocol.SQL.prototype.destroy.apply(this, arguments);
    },
    
    /**
     * Method: initializeDatabase
     */
    initializeDatabase: function() {
        this.db = Giscuit.Util.WebSql.getConn();
        
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "CREATE TABLE IF NOT EXISTS " + this.tableName +
                    " (id INTEGER PRIMARY KEY, feature TEXT)",
                [],
                function() { 
                    this.executeSql(
                        tx,
                        "SELECT MAX(id) as count FROM " + this.tableName,
                        [],
                        function(tx, result) {
                            this.maxId = result.rows.item(0).count || 0;
                        }
                    );
                }
            );
        });
    },
    
    /**
     * Method: transaction
     * Triggers a transaction on the database.
     *
     * Parameters:
     * execute - {Function} The sql transaction callback to execute.
     * failure - {Function} The error callback.
     * success - {Function} The success callback.
     */
    transaction: function(execute, failure, success) {
        if (this.supported) {
            this.db.transaction(
                OpenLayers.Function.bind(execute, this),
                OpenLayers.Function.bind(failure || OpenLayers.Function.Void, this),
                OpenLayers.Function.bind(success || OpenLayers.Function.Void, this)
            );
        } else {
            if (failure) {
                failure.apply(this, ["Protocol not supported"]);
            }
        }
    },
    
    /**
     * Method: executeSql
     * Call transaction.executeSql with callbacks bound to this protocol.
     *
     * Parameters:
     * transaction - {Transaction}
     * sql - {String} The SQL to execute
     * args - {Array} A list of arguments for ? replacement in the sql
     * callback - {Function} The statement callback.
     * failure - {Function} The statement error callback.
     */
    executeSql: function(transaction, sql, args, callback, failure) {
        transaction.executeSql(
            sql, args,
            OpenLayers.Function.bind(callback || OpenLayers.Function.Void, this),
            OpenLayers.Function.bind(failure || OpenLayers.Function.Void, this)
        );
    },

    /**
     * Method: read
     * Read all features from the database and return a
     *     <OpenLayers.Protocol.Response> instance. If the options parameter
     *     contains a callback attribute, the function is called with the response
     *     as a parameter.
     *
     * Parameters:
     * options - {Object} Optional object for configuring the request.
     *
     * Returns:
     * {<OpenLayers.Protocol.Response>} An <OpenLayers.Protocol.Response>
     *      object.
     */
    read: function(options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        var response = new OpenLayers.Protocol.Response({requestType: "read"});
        this.readTransaction(function(tx) {
            this.executeSql(tx, "SELECT * from " + this.tableName, [],
                function(tx, result) {
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },
    
    /**
     * Method: create
     * Construct a request for writing newly created features.
     *
     * Parameters:
     * features - {Array({<OpenLayers.Feature.Vector>})} or
     *     {<OpenLayers.Feature.Vector>}
     * options - {Object} Optional object for configuring the request.
     *
     * Returns:
     * {<OpenLayers.Protocol.Response>} A response object, whose "priv"
     *     property references the transaction, this object is also passed to
     *     the callback function when the request completes.
     */
    create: function(feature, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        var response = new OpenLayers.Protocol.Response({
            reqFeatures: [feature], requestType: "create"
        });
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "INSERT INTO " + this.tableName + " (id, feature) VALUES (?, ?)",
                [this.maxId + 1, this.format.write(feature)],
                function(tx, result) { 
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },

    /**
     * Method: update
     * Construct a request updating modified feature.
     *
     * Parameters:
     * feature - {<OpenLayers.Feature.Vector>}
     * options - {Object} Optional object for configuring the request.
     *     This object is modified and should not be reused.
     *
     * Returns:
     * {<OpenLayers.Protocol.Response>} A response object, whose "priv"
     *     property references the transaction, this object is also passed to
     *     the callback function when the request completes, its "features"
     *     property is then populated with the the feature received from the
     *     database.
     */
    update: function(feature, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        var response = new OpenLayers.Protocol.Response({
            reqFeatures: [feature], requestType: "update"
        });
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "REPLACE INTO " + this.tableName + " (id, feature) VALUES (?, ?)",
                [feature.fid, this.format.write(feature)],
                function(tx, result) { 
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },

    /**
     * Method: delete
     * Construct a request deleting a removed feature.
     *
     * Parameters:
     * feature - {<OpenLayers.Feature.Vector>}
     * options - {Object} Optional object for configuring the request.
     *
     * Returns:
     * {<OpenLayers.Protocol.Response>} A response object, whose "priv"
     *     property references the transaction, this object is also passed to
     *     the callback function when the request completes.
     */
    "delete": function(feature, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        var response = new OpenLayers.Protocol.Response({
            reqFeatures: [feature], requestType: "delete"
        });
        this.transaction(function(tx) {
            this.executeSql(tx, "DELETE FROM " + this.tableName + " where id=?",
                [feature.fid],
                function(tx, result) { 
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },

    /**
     * Method: delete all items from database
     */     
    clear: function(options) {
        options = options || {};
        
        this.transaction(function(tx) {
            this.executeSql(tx, "DELETE FROM " + this.tableName, [], function() {
                if (options.callback) {
                    options.callback.call(options.scope);
                }
            });
        });
    },
        
    /**
     * Method: handleResponse
     * Deals with responses and calls any specified callback.
     *
     * Parameters:
     * response - {OpenLayers.Protocol.Response} The response.
     * options - {Object}
     */
    handleResponse: function(response, options) {
        var result = response.priv.result;
        if (result) {
            switch (response.requestType) {
                case "create":
                    if(result.insertId > this.maxId) {
                        this.maxId = result.insertId;
                    }
                    break;
                case "read":
                    var row, feature;
                    var len = result.rows.length;
                    response.features = new Array(len);
                    for(var i=0; i<len; ++i) {
                        row = result.rows.item(i);
                        feature = this.format.read(row["feature"], "Feature");
                        feature.fid = row["id"];
                        response.features[i] = feature;
                    }
                    break;
            }
            response.code = OpenLayers.Protocol.Response.SUCCESS;
        } else {
            response.code = OpenLayers.Protocol.Response.FAILURE;
        }
        if (options.callback) {
            options.callback.call(options.scope, response);
        }
    },

    /**
     * Method: callUserCallback
     * This method is used from within the commit method each time an
     *     an transaction completes, it is responsible for calling the
     *     user-supplied callbacks.
     *
     * Parameters:
     * resp - {<OpenLayers.Protocol.Response>}
     * options - {Object} The map of options passed to the commit call.
     */
    callUserCallback: function(resp, options) {
        var opt = options[resp.requestType];
        if (opt && opt.callback) {
            opt.callback.call(opt.scope, resp);
        }
    },

    /**
     * Method: drop database
     */
    drop: function(options) {
        this.transaction(function(tx) {
            this.executeSql(tx, "DROP TABLE IF EXISTS " + this.tableName, [], function() {
                if (options.callback) {
                    options.callback.call(options.scope);
                }
            });
        });
    },
    
    /**
     * Method: Read transaction
     * Triggers a read transaction on the database.
     *
     * Parameters:
     * execute - {Function} The sql read transaction callback to execute.
     * failure - {Function} The error callback.
     * success - {Function} The success callback.
     */
    readTransaction: function(execute, failure, success) {
        if (this.supported) {
            this.db.readTransaction(
                OpenLayers.Function.bind(execute, this),
                OpenLayers.Function.bind(failure || OpenLayers.Function.Void, this),
                OpenLayers.Function.bind(success || OpenLayers.Function.Void, this)
            );
        } else {
            if (failure) {
                failure.apply(this, ["Protocol not supported"]);
            }
        }
    },
    
    CLASS_NAME: "OpenLayers.Protocol.SQL.WebSQL"
});   

/**
 * Class: Giscuit.Util.VectorWebSql
 * 
 *  Inherits from:
 *   - <OpenLayers.Protocol.SQL.WebSQL>
 */
Giscuit.Util.WebSql.Vector = OpenLayers.Class(OpenLayers.Protocol.SQL.WebSQL, {
    /**
     * Method: initialCreate
     * Construct a request for writing newly created features.
     */
    initialCreate: function(data, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        
        var jsonFormat = new OpenLayers.Format.JSON();
        
        var response = new OpenLayers.Protocol.Response({
            reqFeatures: [data], requestType: "create"
        });
        
        this.transaction(function(tx) {
            for(var i=0; i < data.length; i++) {
                var id = data[i].properties.gid;
                data[i].properties.__hdate = options.timestamp;
                
                var feature = jsonFormat.write(data[i]);
                
                //FIXME:
                /*
                if(id > this.maxId) {
                    this.maxId = id;
                }
                */
                
                this.executeSql(
                    tx,
                    "INSERT INTO " + this.tableName + " (id, feature) VALUES (?, ?)",
                    [id, feature],
                    function(tx, result) { 
                        response.priv = {transaction: tx, result: result};
                        this.handleResponse(response, options);
                    },
                    function(tx, error) {
                        response.priv = {transaction: tx, error: error};
                        this.handleResponse(response, options);
                    }
                );
            
            }
        }, 
        null,
        function () {
            gt.control.offline.updateFeaturesCount();
        });
        
        return response;
    },
        
    /**
     * Method: handleResponse
     * Deals with responses and calls any specified callback.
     *
     * Parameters:
     * response - {OpenLayers.Protocol.Response} The response.
     * options - {Object}
     */
    handleResponse: function(response, options) {
        var result = response.priv.result;
        if (result) {
            switch (response.requestType) {
                case "create":
                    if(result.insertId > this.maxId) {
                        this.maxId = result.insertId;
                    }
                    break;
                case "read":
                    var row, feature;
                    var len = result.rows.length;
                    response.features = [];
                    for(var i=0; i < len; ++i) {
                        var add = true;
                        
                        row = result.rows.item(i);
                        feature = this.format.read(row["feature"], "Feature");
                        feature.fid = row["id"];
                        
                        //withHStatus
                        if(typeof(options.withHStatus) != 'undefined' && options.withHStatus === true) {
                            if(! feature.attributes.hstatus) {
                                add = false;
                            }
                        } else {
                            if(feature.attributes.hstatus == 'D') {
                                add = false;
                            }
                        }
                        
                        if(add) {
                            response.features.push(feature);
                        }
                    }
                    break;
            }
            response.code = OpenLayers.Protocol.Response.SUCCESS;
        } else {
            response.code = OpenLayers.Protocol.Response.FAILURE;
        }
        if (options.callback) {
            options.callback.call(options.scope, response);
        }
    },
    
    CLASS_NAME: "Giscuit.Util.WebSql.Vector"
});

/**
 * Class: Giscuit.Util.RasterWebSql
 */
Giscuit.Util.WebSql.Raster = OpenLayers.Class(OpenLayers.Protocol.SQL.WebSQL, {
    /**
     * Method: initializeDatabase
     * Initialize database
     */
    initializeDatabase: function() {
        this.db = Giscuit.Util.WebSql.getConn();
        
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "CREATE TABLE IF NOT EXISTS " + this.tableName +
                    " (url TEXT PRIMARY KEY, image TEXT)",
                []
            );
        });
    },
    
    /**
     * Method: read
     * Read all features from the database and return
     */
    read: function(options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        
        //Url
        var urlWhere = '';
        if(options.url) {
            urlWhere = " WHERE url = '" + options.url + "'";
        }
        
        var response = new OpenLayers.Protocol.Response({requestType: "read"});
        this.readTransaction(function(tx) {
            this.executeSql(tx, "SELECT * FROM " + this.tableName + urlWhere, [],
                function(tx, result) {
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },
    
    /**
     * Method: create
     * Construct a request for writing newly created features.
     */
    create: function(url, image, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        var response = new OpenLayers.Protocol.Response({requestType: "create"});
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "INSERT INTO " + this.tableName + " (url, image) VALUES (?, ?)",
                [url, image],
                function(tx, result) { 
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },
    
    /**
     * Method: update
     */
    update: function(feature, options) {
    },

    /**
     * Method: delete
     */
    "delete": function(url, options) {
    },
    
    /**
     * Method: handleResponse
     * Deals with responses and calls any specified callback.
     *
     * Parameters:
     * response - {OpenLayers.Protocol.Response} The response.
     * options - {Object}
     */
    handleResponse: function(response, options) {
        var result = response.priv.result;
        if (result) {
            switch (response.requestType) {
                case "read":
                    var row;
                    var len = result.rows.length;
                    response.images = [];
                    for(var i=0; i<len; ++i) {
                        row = result.rows.item(i);
                        response.images.push({url: row['url'], image: row['image']});
                    }
                    break;
            }
            response.code = OpenLayers.Protocol.Response.SUCCESS;
        } else {
            response.code = OpenLayers.Protocol.Response.FAILURE;
        }
        if (options.callback) {
            options.callback.call(options.scope, response);
        }
    },
        
    CLASS_NAME: "Giscuit.Util.WebSql.Raster"
});

/**
 * Class: Giscuit.Util.FilesWebSql
 */
Giscuit.Util.WebSql.Files = OpenLayers.Class(OpenLayers.Protocol.SQL.WebSQL, {
    /**
     * Method: initializeDatabase
     * Initialize database
     */
    initializeDatabase: function() {
        this.db = Giscuit.Util.WebSql.getConn();
        
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "CREATE TABLE IF NOT EXISTS " + this.tableName +
                    " (id INTEGER PRIMARY KEY, data TEXT, name TEXT)",
                [],
                function() {
                    this.executeSql(
                        tx,
                        "SELECT MAX(id) as count FROM " + this.tableName,
                        [],
                        function(tx, result) {
                            this.maxId = result.rows.item(0).count || 0;
                        }
                    );
                }
            );
        });
    },
    
    /**
     * Method: read
     * Read all features from the database and return
     */
    read: function(options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        
        var where = '';
        var fields = '*';
        
        //where options
        if (options.id) {
            where += " WHERE id = " + options.id;
        }
        
        var response = new OpenLayers.Protocol.Response({requestType: "read"});
        this.readTransaction(function(tx) {
            this.executeSql(tx, "SELECT " + fields + " FROM " + this.tableName + where, [],
                function(tx, result) {
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },
    
    /**
     * Method: create
     * Construct a request for writing newly created features.
     */
    create: function(data, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        var response = new OpenLayers.Protocol.Response({requestType: "create"});
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "INSERT INTO " + this.tableName + " (id, data, name) VALUES (?, ?, ?)",
                [data.id, data.data, data.name],
                function(tx, result) { 
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },
    
    /**
     * Method: update
     */
    update: function(values, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        
        var where = '';
        
        
        //Id
        if (options.id) {
            where = " WHERE id = " + options.id;
        }
        
        //Values
        var valuesStr = '';
        var valuesArr = [];
        for (var key in values) {
            valuesStr += key + '=?,';
            valuesArr.push(values[key]);
        }
        valuesStr = valuesStr.slice(0, -1);
        
        var response = new OpenLayers.Protocol.Response({requestType: "update"});
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "UPDATE " + this.tableName + " SET " + valuesStr + where,
                valuesArr,
                function(tx, result) { 
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },

    /**
     * Method: delete
     */
    "delete": function(id, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        var response = new OpenLayers.Protocol.Response({requestType: "delete"});
        this.transaction(function(tx) {
            this.executeSql(
                tx, "DELETE FROM " + this.tableName + " WHERE id=?",
                [id],
                function(tx, result) { 
                    response.priv = {transaction: tx, result: result};
                    this.handleResponse(response, options);
                },
                function(tx, error) {
                    response.priv = {transaction: tx, error: error};
                    this.handleResponse(response, options);
                }
            );
        });
        return response;
    },
    
    /**
     * Method: getMinId
     * Get minimum id value from the database and return
     */
    getMinId: function(callback) {
        this.readTransaction(function(tx) {
            this.executeSql(tx, "SELECT MIN(id) AS minId FROM " + this.tableName, [],
                function(tx, result) {
                    callback(result.rows.item(0)['minId']);
                },
                Giscuit.Util.WebSql.error
            );
        });
    },
    
    /**
     * Method: handleResponse
     * Deals with responses and calls any specified callback.
     *
     * Parameters:
     * response - {OpenLayers.Protocol.Response} The response.
     * options - {Object}
     */
    handleResponse: function(response, options) {
        var result = response.priv.result;
        if (result) {
            switch (response.requestType) {
                case "create":
                    if(result.insertId > this.maxId) {
                        this.maxId = result.insertId;
                    }
                    break;
                case "read":
                    var row, len = result.rows.length;
                    response.files = [];
                    
                    if (typeof(options.record) != 'undefined') {
                        response.record = options.record;
                    }

                    for (var i=0; i<len; ++i) {
                        row = result.rows.item(i);
                        response.files.push({
                            id: row['id'], 
                            data: row['data'], 
                            name: row['name']
                        });
                    }
                    
                    break;
            }
            response.code = OpenLayers.Protocol.Response.SUCCESS;
        } else {
            response.code = OpenLayers.Protocol.Response.FAILURE;
        }
        if (options.callback) {
            options.callback.call(options.scope, response);
        }
    },
        
    CLASS_NAME: "Giscuit.Util.WebSql.Files"
});

/**
 * Function: Giscuit.Util.WebSql.getConn
 */
Giscuit.Util.WebSql.getConn = function () {
    if(! Giscuit.Util.WebSql.dbConn) {
        Giscuit.Util.WebSql.dbConn = openDatabase(
            Giscuit.Util.WebSql.name, 
            Giscuit.Util.WebSql.version, 
            Giscuit.Util.WebSql.desc, 
            Giscuit.Util.WebSql.size
        );
        
        if(! Giscuit.Util.WebSql.dbConn) {
            Giscuit.Util.WebSql.dbConn = openDatabase(
                Giscuit.Util.WebSql.name, 
                Giscuit.Util.WebSql.version, 
                Giscuit.Util.WebSql.desc, 
                5 * 1024 * 1024
            );
        }
    }
    return Giscuit.Util.WebSql.dbConn;
};

/**
 * Function: Giscuit.Util.WebSql.getMetadata
 */
Giscuit.Util.WebSql.getMetadata = function(callback) {
    var db = Giscuit.Util.WebSql.getConn();
    var sql = "SELECT name FROM sqlite_master WHERE type='table' AND name != '__WebKitDatabaseInfoTable__'";
    
    db.readTransaction(function (tx) {
        tx.executeSql(sql, [], function (tx, results) {
           var countSQL = [];
           var tables = {};
            
           var len = results.rows.length, i;
           for (i = 0; i < len; i++) {
               var name = results.rows.item(i).name;
               countSQL.push("SELECT '" + name + "' as name, count(*) as count FROM " + name);
           }

           if(countSQL.length == 0) {
               callback(tables);
               return;
           }

           countSQL = countSQL.join(' UNION ');
           
           tx.executeSql(countSQL, [], function (tx, results) {
               var len = results.rows.length, i;
               for (i = 0; i < len; i++) {
                   var id = results.rows.item(i).name;
                   id = id.split('_')[1];
                   if(! id) {
                       continue;
                   }
                   
                   tables[id] = {
                       'name': results.rows.item(i).name,
                       'count': results.rows.item(i).count
                   };
               }

               callback(tables);
               
           }, Giscuit.Util.WebSql.error);
           
        }, Giscuit.Util.WebSql.error);
    });
};

/**
 * Function: Giscuit.Util.WebSql.error
 */
Giscuit.Util.WebSql.error = function (tx, evt) {
    Giscuit.Util.jqmMessage(evt.message);
    debug.warn(evt.message);
};